﻿"""
/***************************************************************************
Name                  : Mask 
Description          : Aide à la création de masque
Date                 : Feb/12 
copyright            : (C) 2011 by AEAG
email                : xavier.culos@eau-adour-garonne.fr 
todo: 

 add help file
 let's user change user defined parameters. 
 use a qml file to set masqk style. default qml inside plugin ? 
   Parameter 1: choose memory output or shp output + directory (default HOME directory)
   Parameter 2 : choose a qml file. Qml should be parsed to check if qymbology apply to polygon features

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
import os
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from qgis.core import *

# Initialize Qt resources from file resources.py
import resources

aeag_mask_instance = None

def do(workOnSelectedFeatures=True, extent=None):
    global aeag_mask_instance
    aeag_mask_instance.do_aeag_mask(workOnSelectedFeatures, extent);

class aeag_mask: 

    def __init__(self, iface):
        global aeag_mask_instance
        # Save reference to the QGIS interface
        self.iface = iface
        self.toolBar = None
        self.act_aeag_mask = None
        self.act_aeag_toolbar_help = None
        self.canvas = self.iface.mapCanvas()
        self.active = False
        self.read()
        aeag_mask_instance = self
      
    def store(self):
        s = QSettings() 
    
    def read(self):
        s = QSettings()

    def initGui(self):  
        self.toolBar = self.iface.pluginToolBar()
        
        self.act_aeag_mask = QAction(QIcon(":plugins/mask/aeag_mask.png"), ("Create mask"), self.iface.mainWindow())
        self.act_aeag_mask.setCheckable(True)
        self.toolBar.addAction(self.act_aeag_mask)
        
        self.iface.addPluginToMenu("&Mask", self.act_aeag_mask)    
        
        # Add actions to the toolbar
        self.act_aeag_mask.triggered.connect(self.do_aeag_mask)
        
        #add a link to helpfile for toolbar & menu aeag , if not exist


    def unload(self):
        self.toolBar.removeAction(self.act_aeag_mask)
        self.iface.removePluginMenu("&Mask", self.act_aeag_mask)
        self.store()

    # run method that performs all the real work
    def do_aeag_mask(self, unionFeatures=True, extent=None): 
        #QMessageBox.information(None, "Cancel", "do_aeag_mask")
        self.active = True
        
        #Begin lines edited by Caio Hamamura
        #Remove previous Mask layer
        [QgsMapLayerRegistry.instance().removeMapLayer(i.id()) for i in self.iface.legendInterface().layers() if i.name() == 'Mask']
        #End lines edited by Caio Hamamura
        
        layermap = QgsMapLayerRegistry.instance().mapLayers()

        #Begin lines edited by Caio Hamamura
        #Track previous active layer
        activeLayer = self.iface.activeLayer()
        #End lines edited by Caio Hamamura
        
        if extent == None:
            #Begin lines edited by Caio Hamamura
            #Changed to full extent instead of current extent
            rect = self.canvas.fullExtent()
            #End lines edited by Caio Hamamura
            #rect.scale(2)
        else:
            rect = extent
        
        if unionFeatures:
            g = None
            theLayer = None
            layerName = "Mask"
            for name, layer in layermap.iteritems():
                if layer.type() == QgsMapLayer.VectorLayer:
                    if layer.isValid():
                        theLayer = layer
                        features = layer.selectedFeatures()
                        for feature in features:
                            if feature.geometry() and feature.geometry().type() == QGis.Polygon:
                                if g == None:
                                    g = feature.geometry()
                                    f = feature
                                    try:
                                        layerName = "Mask " + feature.attributeMap()[layer.dataProvider().fieldNameIndex(layer.displayField())]
                                    except:
                                        layerName = "Mask"
                                else:
                                    g = g.combine(feature.geometry())
                                    layerName = "Mask"
                                    
                    layer.removeSelection()  # remove user selection after use (clean screen from yellow display)
                    
            if g != None:
                maskLayer = QgsVectorLayer("MultiPolygon", layerName, "memory")

                # map crs
                mask = QgsGeometry.fromRect(rect) 
            
                layerCrs = theLayer.crs();
                mapCrs = self.canvas.mapRenderer().destinationCrs()
                maskCrs = maskLayer.crs();
                xLayer2Mask = QgsCoordinateTransform(layerCrs, maskCrs)
                xMap2Mask = QgsCoordinateTransform(mapCrs, maskCrs)
                xMask2Map = QgsCoordinateTransform(maskCrs, mapCrs)
                # mask in map CRS
                mask.transform(xMap2Mask)
                # g in initial layer CRS
                g.transform(xLayer2Mask)
                mask.makeDifference(g)
                
                #Begin lines edited by Caio Hamamura
                #Put mask layer on the top of everything else
                self.iface.setActiveLayer(self.iface.legendInterface().layers()[0])
                QgsMapLayerRegistry.instance().addMapLayer(maskLayer)
                self.iface.legendInterface().setLayerVisible(maskLayer, True)
                self.iface.setActiveLayer(activeLayer)
                #End lines edited by Caio Hamamura
            
                pr = maskLayer.dataProvider()
                fet = QgsFeature()
                fet.setGeometry(mask)
                pr.addFeatures([ fet ])
                maskLayer.updateExtents()        
            
                # style
                rendererV2 = maskLayer.rendererV2()
                for s in rendererV2.symbols():
                    s.setAlpha(0.90)
                    s.setColor(QColor(255, 255, 255))
                    #Begin lines edited by Caio Hamamura
                    #Remove mask border
                    s.symbolLayer(0).setBorderStyle(0)
                    #End lines edited by Caio Hamamura
                        
                    
                # Refresh legend
                self.iface.legendInterface().refreshLayerSymbology( maskLayer ) 
                QgsMapLayerRegistry.instance().clearAllLayerCaches () #clean cache to allow mask layer to appear on refresh
                
                #Begin lines edited by Caio Hamamura
                #Zoom to selected features (Map CRS)
                g.transform(xMask2Map)
                self.canvas.setExtent(g.boundingBox())
                #self.canvas.zoomToNextExtent()
                #End lines edited by Caio Hamamura
                self.canvas.refresh()
               
            else:
                QMessageBox.information(None, "Attention", ("A selection of polygons is needed..."))

        # one mask for each feature
        else:
            layerName = "Masque"
            layers = self.iface.legendInterface().selectedLayers()
            for layer in layers:
                if layer.type() == QgsMapLayer.VectorLayer:
                    if layer.isValid():
                        maskLayer = QgsVectorLayer("MultiPolygon", layerName+"_"+layer.name(), "memory")

                        layer.selectAll()
                        
                        sourceProvider = layer.dataProvider()
                        
                        layerCrs = layer.crs();
                        mapCrs = self.canvas.mapRenderer().destinationCrs()
                        maskCrs = maskLayer.crs();
                        xLayer2Mask = QgsCoordinateTransform(layerCrs, maskCrs)
                        xMap2Mask = QgsCoordinateTransform(mapCrs, maskCrs)
                        xMask2Map = QgsCoordinateTransform(maskCrs, mapCrs)

                        #Begin lines edited by Caio Hamamura
                        #Put mask layer on the top of everything else
                        self.iface.setActiveLayer(self.iface.legendInterface().layers()[0])
                        QgsMapLayerRegistry.instance().addMapLayer(maskLayer)
                        self.iface.legendInterface().setLayerVisible(maskLayer, True)
                        self.iface.setActiveLayer(activeLayer)
                        #End lines edited by Caio Hamamura
                        pr = maskLayer.dataProvider()
                        
                        # attributes
                        for f in sourceProvider.fields():
                            pr.addAttributes( [ QgsField(f.name(), f.type()) ] )

                        for feature in layer.selectedFeatures():
                            if feature.geometry().type() == QGis.Polygon:
                                g = feature.geometry()
                                
                                mask = QgsGeometry.fromRect(rect)
                                
                                # mask in map CRS
                                mask.transform(xMap2Mask)
                                # g in initial layer CRS
                                g.transform(xLayer2Mask)
                                mask.makeDifference(g)

                                fet = QgsFeature(feature)
                                fet.setGeometry(mask)
                                pr.addFeatures([ fet ])
                                    
                        # / features
                        
                        # style
                        rendererV2 = maskLayer.rendererV2()
                        for s in rendererV2.symbols():
                            s.setAlpha(0.90)
                            s.setColor(QColor(255, 255, 255))
                            #Begin lines edited by Caio Hamamura
                            #Remove mask border
                            s.symbolLayer(0).setBorderStyle(0)
                            #Begin end edited by Caio Hamamura

                        #Refresh legend
                        layer.removeSelection()         
                        maskLayer.updateExtents()        
                        self.iface.legendInterface().refreshLayerSymbology( maskLayer ) 
                        QgsMapLayerRegistry.instance().clearAllLayerCaches () #clean cache to allow mask layer to appear on refresh

                        self.canvas.refresh()
                    else:
                        print "Invalid layer"
                else:
                    print "Selected layer is not VectorLayer"
        
        self.act_aeag_mask.setChecked(False)
        self.active = False
    
    def deactivate(self):
        self.act_aeag_mask.setChecked(False)
        self.active = False
